#!/usr/bin/env python


import time

from common import Common


class InjectionCheck(Common):
    def __check_sqlinj(self, parameter, value):
        """
        Check if the url parameter is sql injectable and identifies
        the type of sql injection:

            * Numeric/Unescaped injection
            * String/Single quote injection
            * String/Double quotes injection
        """

        self.log("testing numeric/unescaped injection on parameter '%s'" % parameter)
        url = self.url_replace_parameter(parameter, value, '%s AND 1=1' % value)
        true_hash = self.hash_page(url)

        if true_hash == self.args.default_hash:
            url = self.url_replace_parameter(parameter, value, '%s AND 1=2' % value)
            self.false_hash = self.hash_page(url)

            if self.false_hash != self.args.default_hash:
                self.log("confirming numeric/unescaped injection on parameter '%s'" % parameter)
                url = self.url_replace_parameter(parameter, value, '%s AND NoValue' % value)
                self.false_hash = self.hash_page(url)

                if self.false_hash != self.args.default_hash:
                    self.log("parameter '%s' is numeric/unescaped injectable" % parameter)
                    self.args.injection_method = 'numeric'
                    return True

        self.log("parameter '%s' is not numeric/unescaped injectable" % parameter)

        self.log("testing string/single quote injection on parameter '%s'" % parameter)
        url = self.url_replace_parameter(parameter, value, "%s' AND '1'='1" % value)
        true_hash = self.hash_page(url)

        if true_hash == self.args.default_hash:
            url = self.url_replace_parameter(parameter, value, "%s' AND '1'='2" % value)
            self.false_hash = self.hash_page(url)

            if self.false_hash != self.args.default_hash:
                self.log("confirming string/single quote injection on parameter '%s'" % parameter)
                url = self.url_replace_parameter(parameter, value, "%s' and NoValue" % value)
                self.false_hash = self.hash_page(url)

                if self.false_hash != self.args.default_hash:
                    self.log("parameter '%s' is string/single quote injectable" % parameter)
                    self.args.injection_method = 'stringsingle'
                    return True

        self.log("parameter '%s' is not string/single quote injectable" % parameter)

        self.log("testing string/double quotes injection on parameter '%s'" % parameter)
        url = self.url_replace_parameter(parameter, value, '%s" AND "1"="1' % value)
        true_hash = self.hash_page(url)

        if true_hash == self.args.default_hash:
            url = self.url_replace_parameter(parameter, value, '%s" AND "1"="2' % value)
            self.false_hash = self.hash_page(url)

            if self.false_hash != self.args.default_hash:
                self.log("confirming string/double quotes injection on parameter '%s'" % parameter)
                url = self.url_replace_parameter(parameter, value, '%s" AND "NoValue' % value)
                self.false_hash = self.hash_page(url)

                if self.false_hash != self.args.default_hash:
                    self.log("parameter '%s' is string/double quotes injectable" % parameter)
                    self.args.injection_method = 'stringdouble'
                    return True

        self.log("parameter '%s' is not string/double quotes injectable" % parameter)

        return False


    def __check_dynamics_param(self, parameter, value):
        """
        Check if the url parameter is dynamic. If it is dynamic, the
        content of the page differs, otherwise the dynamicity might
        depend on another parameter.
        """

        self.args.default_hash = self.hash_page(self.args.url)

        url = self.url_replace_parameter(parameter, value, '47')
        dynamics_hash = self.hash_page(url)

        if self.args.default_hash == dynamics_hash:
            return False

        self.log("confirming that '%s' parameter is dynamic" % parameter)

        url = self.url_replace_parameter(parameter, value, "'NoValue")
        dynamics1_hash = self.hash_page(url)

        url = self.url_replace_parameter(parameter, value, '"NoValue')
        dynamics2_hash = self.hash_page(url)

        return self.args.default_hash != dynamics1_hash or self.args.default_hash != dynamics2_hash


    def __check_url_stable(self):
        """
        Check if the url is stable requesting the original url page
        three times with a small delay within each request to assume
        it is.

        In case the content of the page differs when requesting the
        same page, the dynamicity might depend on other settings.
        """

        first_hash = self.hash_page(self.args.url)
        time.sleep(0.5)
        second_hash = self.hash_page(self.args.url)
        time.sleep(0.5)
        third_hash = self.hash_page(self.args.url)

        return first_hash == second_hash and second_hash == third_hash


    def check(self, checkedArgs):
        """
        Perform checks on the url:

            * Check if url is stable requesting the original url
              three times;
            * Parse all parameters, identify the first dynamic one
              and check if it is sql injectable, if it's not injectable
              pass to check the next dynamic parameter.
        """

        self.args = checkedArgs
        self.args.injparameter = None

        self.log("testing if the url is stable, wait a few seconds")
        if not self.__check_url_stable():
            err_msg  = "ERROR: url is not stable, unable to "
            err_msg += "perform blind sql injection"
            raise Exception, err_msg
        else:
            self.log("url is stable")

        for parameter, value in self.args.parameters.items():
            self.log("testing if '%s' parameter is dynamic" % parameter)
            if not self.__check_dynamics_param(parameter, value):
                self.warn("parameter '%s' is not dynamic" % parameter)
            else:
                self.log("parameter '%s' is dynamic" % parameter)

                self.log("testing sql injection on parameter '%s'" % parameter)
                if not self.__check_sqlinj(parameter, value):
                    self.warn("parameter '%s' is not injectable" % parameter)
                else:
                    self.args.injparameter = parameter
                    break

        return self.args
